// SQLiteManager extension
var sQuerySelectInstruction = "(Select a Query)";
var Database = new SQLiteHandler();

var smStructTrees = [];
smStructTrees[0] = new TreeDbStructure("t-dbStructNorm", "tc-dbStructNorm", 0);
smStructTrees[1] = new TreeDbStructure("t-dbStructMstr", "tc-dbStructMstr", 1);
smStructTrees[2] = new TreeDbStructure("t-dbStructTemp", "tc-dbStructTemp", 2);

var treeBrowse = new TreeDataTable("browse-tree");
var treeExecute = new TreeDataTable("treeSqlOutput");

var smExtManager = null;

var SQLiteManager = {    
  prefs: null,

  miDbObjects: 0,
  //for display in the browse tree
  miLimit: -1,
  miOffset: 0,
  miCount: 0,
  msBrowseObjName: null,
  msBrowseCondition: null,
  
  // Currently selected database file (nsIFile)
  sCurrentDatabase: null, 
  
  //an array containing names of current table, index, view and trigger
  aCurrObjNames: [],
  
  //to store the latest selection in tree showing db objects
  mostCurrObjName: null,
  mostCurrObjType: null,
  
  mbDbJustOpened: true,
  // an array of 4 arrays; 
  // each holding names of tables, indexes, views and triggers
  aObjNames: [],
  aObjTypes: ["table","view","index","trigger"],

  clipService: null,      // Clipboard service: nsIClipboardHelper

  // Status bar: panels for displaying various info
  sbPanel: [],         

  //store extension pertaining mgmt info externally/internally
  m_bMgmtDataIsInDb: true,

  maFileExt: [],
  //whether to openDatabase or openUnsharedDatabase
  mbSharedPagerCache: true,

  experiment: function() {
  },

  // Startup: called ONCE during the browser window "load" event
  Startup: function() {
    //create the menus by associating appropriate popups
    this.createMenu();

    g_smBundle = document.getElementById("sm-bundle");

    //initialize the structure tree
 		smStructTrees[0].init();
 		smStructTrees[1].init();
 		smStructTrees[2].init();
    this.refreshDbStructure();

    treeBrowse.init();
    treeExecute.init();

    this.setSharedPagerCache();
    //enable pragma version options for gecko 1.9
    if (gbGecko_1_9b3)
      document.getElementById("gb-pragmaVersion").hidden = false;

    // Load clipboard service
    this.clipService= Components
    	.classes["@mozilla.org/widget/clipboardhelper;1"]
       .getService(Components.interfaces.nsIClipboardHelper);

    //get the nodes for Status bar panels
    this.sbPanel["display"] = document.getElementById("sbPanel-display");
    //global var in misc.js for sbpanel[display]
    gSbPanelDisplay = this.sbPanel["display"];

    //display whether we are using Gecko 1.9+ in the status bar (Issue#35)
    var sbGeckoVersion = document.getElementById("sbGeckoVersion");
    sbGeckoVersion.setAttribute("label","Gecko " + gAppInfo.platformVersion);
    if (gbGecko_1_9b3)
    	sbGeckoVersion.setAttribute("style","font-weight:bold;color:green");

		document.getElementById("vb-structureTab").hidden = true;
		document.getElementById("vb-browseTab").hidden = true;
		document.getElementById("vb-executeTab").hidden = true;
		document.getElementById("vb-dbInfoTab").hidden = true;

    //preferences service to add oberver
    // and then observe changes via the observe function
    //see http://developer.mozilla.org/en/docs/Adding_preferences_to_an_extension
    //initialize the preference service with the correct branch
    this.prefs = sm_prefsBranch;
    //query interface to be able to use addObserver method
    this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
    //now, add the observer which will be implemented using observe method
    //calling removeObserver when done with observing helps the memory
    this.prefs.addObserver("", this, false);

    //manage changes when the following preferences change.
    //although the this.observe function handles the observer
    //calling these functions reduces the size of the code
    //It is like fooling observe into believing that the following
    //preferences have changed.
    var chPrefs = ["hideMainToolbar", "showMainToolbarDatabase", "showMainToolbarTable", "showMainToolbarIndex", "sqliteFileExtensions", "displayNumRecords", "textForBlob", "showBlobSize", 
    "posInTargetApp" /* this last one for firefox only*/ ];
    for(var i = 0; i < chPrefs.length; i++)
      this.observe("", "nsPref:changed", chPrefs[i]);

    // opening with last used DB if preferences set to do so
    var bPrefVal = sm_getPreferenceValue("openWithLastDb", false);
		if(bPrefVal) {
			var sPrefVal = sm_getPreferenceValue("mruPath.1", "");
			if(sPrefVal.length > 0)
			{
				//Last used DB found, open this DB
				var newfile = Components.classes["@mozilla.org/file/local;1"]
									.createInstance(Components.interfaces.nsILocalFile);
				newfile.initWithPath(sPrefVal);
				//if the last used file is not found, bail out
				if(!newfile.exists()) {
					alert(sm_getLStr("lastDbDoesNotExist") + sPrefVal);
				  return;
				}
				bPrefVal = sm_getPreferenceValue("promptForLastDb", true);
				if(bPrefVal) {
					var check = {value: false}; // default the checkbox to false
					var result = smPrompt.confirmCheck(null, sm_getLStr("promptLastDbTitle"),
					       sm_getLStr("promptLastDbAsk")+ "\n" + sPrefVal + "?",
                 sm_getLStr("promptLastDbOpen"), check);

					if(!result)
						return;
					//update the promptForLastDb preference
					bPrefVal = sm_setPreferenceValue("promptForLastDb", !check.value);
				}
				//assign the new file (nsIFile) to the current database
				this.sCurrentDatabase = newfile;
				this.setDatabase(this.sCurrentDatabase);
			}
		}
		//load the previously opened tab
    this.loadTabWithId(this.getSelectedTabId());
		return;
  },

  // Shutdown: called ONCE during the browser window "unload" event
  Shutdown: function() {
    //close the current database
    this.closeDatabase(false);
    //Destruction - this should be fired once you're done observing
    //Failure to do so may result in memory leaks.	
    this.prefs.removeObserver("", this);

    this.clipService= null;
  },  

	createMenu: function() {
		var suffixes = ["table", "index", "view", "trigger"];
		for(var i = 0; i < suffixes.length; i++) {
		  var suffix = suffixes[i];
		  var mp = document.getElementById("menupopup-" + suffix);
		  var clone = mp.cloneNode(true);
		  clone.setAttribute("id", "menupopup-main-" + suffix);
		  var menu = document.getElementById("menu-" + suffix);
		  ClearElement(menu);
		  menu.appendChild(clone);
		}
	},

	changeDbSetting: function(sSetting) {
		var node = document.getElementById("pr-" + sSetting);
		var sVal = node.value;
		var newVal = Database.setSetting(sSetting, sVal);
		node.value = newVal;
	},
		
	setTreeStructureContextMenu: function(event) {
    var tc = document.getElementById(smStructTrees[this.miDbObjects].treeChildrenId);
		var tree = document.getElementById(smStructTrees[this.miDbObjects].treeId);

    tc.setAttribute("context","");
    if (this.miDbObjects == 1) //no context menu for master tables
      return;
		var idx = tree.currentIndex;
		// idx = -1 if nothing is selected; says xulplanet element reference
		if(idx == -1)
			idx = 0;
		var objName = tree.view.getCellText(idx, tree.columns.getColumnAt(0));
		var level = tree.view.getLevel(idx);
 		var info = smStructTrees[this.miDbObjects].getSmType(idx);
		//there is a database object at level 1 only
		if (level == 0) {
  		if(info.indexOf("all-") == 0) {
        info = info.substring("all-".length).toLowerCase();
  			if (this.aObjTypes.indexOf(info) != -1)
  				tc.setAttribute("context","menupopup-create-" + info);
			}
		}	
		if (level == 1) {
      info = info.toLowerCase();
 			if (this.aObjTypes.indexOf(info) != -1)
   		  tc.setAttribute("context","menupopup-" + info);
  	}
	},
	
	observe: function(subject, topic, data) {
		if (topic != "nsPref:changed")
			return;
		
		switch(data) {
			case "hideMainToolbar":
				var bPrefVal = sm_prefsBranch.getBoolPref("hideMainToolbar");
				document.getElementById("hbox-main-toolbar").hidden = bPrefVal;
				break;
			case "showMainToolbarDatabase":
				var bPrefVal = sm_prefsBranch.getBoolPref("showMainToolbarDatabase");
				document.getElementById("sm-toolbar-database").hidden = !bPrefVal;
				break;
			case "showMainToolbarTable":
				var bPrefVal = sm_prefsBranch.getBoolPref("showMainToolbarTable");
				document.getElementById("sm-toolbar-table").hidden = !bPrefVal;
				break;
			case "showMainToolbarIndex":
				var bPrefVal = sm_prefsBranch.getBoolPref("showMainToolbarIndex");
				document.getElementById("sm-toolbar-index").hidden = !bPrefVal;
				break;
			case "sqliteFileExtensions":
				var sExt = sm_prefsBranch.getCharPref("sqliteFileExtensions");
				this.maFileExt = sExt.split(",");
        // Load profile folder's sqlite db files list into dropdown 
				this.populateProfileDBList();   
				break;
			case "searchToggler":
				var sPrefVal = sm_prefsBranch.getCharPref("searchCriteria");
				this.msBrowseCondition = sPrefVal;
				//empty the criteria after use for security
				sm_prefsBranch.setCharPref("searchCriteria", "");
				this.loadTabBrowse();
				break;
			case "displayNumRecords":
				var iPrefVal = sm_prefsBranch.getIntPref("displayNumRecords");
				this.miLimit = iPrefVal;
				this.miOffset = 0;
				break;
			case "textForBlob":
				g_strForBlob = sm_getPreferenceValue("textForBlob", g_strForBlob);
				break;
			case "showBlobSize":
				g_showBlobSize = sm_getPreferenceValue("showBlobSize", g_showBlobSize);
				break;
			case "posInTargetApp":
      var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
                        .getService(Components.interfaces.nsIXULAppInfo);
      if(appInfo.ID == "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}") {
        var md = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
          .getInterface(Components.interfaces.nsIWebNavigation)
          .QueryInterface(Components.interfaces.nsIDocShellTreeItem).rootTreeItem
          .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
          .getInterface(Components.interfaces.nsIDOMWindow).document;
        var iVal = sm_prefsBranch.getIntPref("posInTargetApp");
        var mi = md.getElementById("menuitem-sqlitemanager");
        if (mi) {
          if (iVal == 0)
            mi.setAttribute("hidden", true);
          if (iVal == 1)
            mi.setAttribute("hidden", false);
        }
      }
		}
	},
		 
  refresh: function() {
    if (this.sCurrentDatabase == null)
    	return false;
    this.refreshDbStructure();
    return true; 
  },
  //Issue #108
  reconnect: function() {
    var nsFile = this.sCurrentDatabase;
    this.closeDatabase(false);

    this.sCurrentDatabase = nsFile;
    this.setDatabase(this.sCurrentDatabase);  
  },

  //refreshDbStructure: populates the schematree based on selected database
  //must be called whenever a database is opened/closed
  //and whenever the schema changes
  refreshDbStructure: function() {
    //1. if no database is selected
    if (this.sCurrentDatabase == null) {
  		smStructTrees[0].removeChildData();
  		smStructTrees[1].removeChildData();
  		smStructTrees[2].removeChildData();

  		for(var i = 0; i < this.aObjTypes.length; i++) {
  			var type = this.aObjTypes[i];
  			this.aCurrObjNames[type] = null;
  		}
  		return;
  	}

    //2. if db is being opened, set nodes to expand
    if (this.mbDbJustOpened) {
      //set the expandable nodes here
      var aExpand = [["all-table"],[]];
  		//check whether aExpand data is in smextmgmt table and use it
		  if (smExtManager.getUsage()) {
		    aExpand = smExtManager.getStructTreeState();
		  }
	    smStructTrees[0].setExpandableNodes(aExpand);
	    smStructTrees[1].setExpandableNodes([["all-table"],[]]);
	    smStructTrees[2].setExpandableNodes([["all-table"],[]]);
    }

    //3. 
    var tree = document.getElementById(smStructTrees[this.miDbObjects].treeId);

		//requery for all the objects afresh and redraw the tree
		this.aObjNames["table"] = Database.getObjectList("table", this.miDbObjects);
		this.aObjNames["index"] = Database.getObjectList("index", this.miDbObjects);
		this.aObjNames["view"] = Database.getObjectList("view", this.miDbObjects);
		this.aObjNames["trigger"] = Database.getObjectList("trigger", this.miDbObjects);

		var idx = tree.currentIndex;
    smStructTrees[this.miDbObjects].setChildData(this.aObjNames);

		tree.view.selection.select(idx); //triggers getDbObjectInfo function
		
		//now assign the current objects
		for(var i = 0; i < this.aObjTypes.length; i++) {
			var type = this.aObjTypes[i];
			if(this.aObjNames[type].length > 0) {
				var bFound = false;
				if(this.aCurrObjNames[type]) {
					for(var n = 0; n < this.aObjNames[type].length; n++) {
						if(this.aCurrObjNames[type] == this.aObjNames[type][n]) {
							bFound = true;
							break;
						}
					}
				}
				if(!bFound)
					this.aCurrObjNames[type] = this.aObjNames[type][0];
			}
			else
				this.aCurrObjNames[type] = null;
		}
  },
		 		 
	//getDbObjectInfo: this function must show the structural info about the
	// selected database object (table, index, view & trigger)
	//this function is trigered by the select event on the tree
  getDbObjectInfo: function() {
    var tree = document.getElementById(smStructTrees[this.miDbObjects].treeId);
		var idx = tree.currentIndex;

		// idx = -1 if nothing is selected; says xulplanet element reference
		if(idx < 0 || idx >= tree.view.rowCount)
			idx = 1; //first table

		var level = tree.view.getLevel(idx);

		//there is a database object at level 1 only
		if(level == 0) {
			this.mostCurrObjName = null;
			this.mostCurrObjType = null;
			return false;
		}
		//level 2 is a field name of the parent table
		if(level == 2) {
		  idx = tree.view.getParentIndex(idx);
		}	
		var r_name = tree.view.getCellText(idx, tree.columns.getColumnAt(0));
    var r_type = smStructTrees[this.miDbObjects].getSmType(idx);
			
		//assign current selection in tree as current object
		this.aCurrObjNames[r_type] = r_name;

		this.mostCurrObjName = r_name;
		this.mostCurrObjType = r_type;

		this.loadTabStructure();
		this.loadTabBrowse();

		return true;
  },

  hideTabStructure: function() {
		//hide the hboxes containing object specific operation buttons
		//later enable one appropriate hbox according to the selection in tree
		for(var i=0; i < this.aObjTypes.length; i++) {
			document.getElementById("gb-master-ops-" + this.aObjTypes[i])
			  .hidden = true;
			document.getElementById("gb-more-info-" + this.aObjTypes[i])
			  .hidden = true;
		}			
    //hide other things
		document.getElementById("gb-master-info").hidden = true;
  },

  emptyTabStructure: function() {
		//hide the hboxes containing object specific operation buttons
		//later enable one appropriate hbox according to the selection in tree
		this.hideTabStructure();

		var fld = ["type","name","tbl_name","rootpage", "sql"];
		var node;
		for (var i = 0; i < fld.length; i++) {
			node = document.getElementById("str-" + fld[i]);
			node.value = "";
		}

    this.printTableInfo(null);
  },

  loadTabStructure: function() {
    //no need to waste resources if this tab is not selected
    if(this.getSelectedTabId() != "tab-structure")
    	return false;
    	
    this.hideTabStructure();

    if (this.sCurrentDatabase == null)
    	return false;

		//there is a database object at level 1 only
		if(this.mostCurrObjName == null) {
			return false;
		}	
		
		//display this groupbox only if it will show something meaningful
		document.getElementById("gb-master-info").hidden = false;
		
		var r_name = this.mostCurrObjName;
		var r_type = this.mostCurrObjType;
		if (this.miDbObjects != 1) //do not enable for master tables
		  document.getElementById("gb-master-ops-" + r_type).hidden = false;

		var fld = ["type","name","tbl_name","rootpage", "sql"];
		var row = Database.getMasterInfo(r_name, fld, this.miDbObjects);
		var node;
		for(var i=0; i < fld.length; i++) {
			node = document.getElementById("str-"+fld[i]);
			var val = row[i];
			if(fld[i] == "type") {
				val = val.toUpperCase();
			}
			node.value = val;
			node.setAttribute("style", "font-weight:bold;font-size:12px;");
		}		

		//do the following for table/index
		if(r_type == "table") {
			var box = document.getElementById("gb-more-info-" + r_type);
			box.hidden = false;
			this.printTableInfo(this.aCurrObjNames[r_type]);
		}
		if(r_type == "index") {
			var box = document.getElementById("gb-more-info-" + r_type);
			box.hidden = false;
			this.printIndexInfo(this.aCurrObjNames[r_type]);
		}
		return true;
  },

  printTableInfo: function(sTable) {
		var vbox = document.getElementById("vb-more-info-table-columns");
		ClearElement(vbox);
    if (sTable == null)
      return;

    document.getElementById("hb-addcol").hidden = false;
    if (this.miDbObjects == 1) //no add column option for master tables
      document.getElementById("hb-addcol").hidden = true;

		var info = Database.getTableColumns(sTable);

		var cols = info[0];
		for(var i = 0; i < cols.length; i++) {
			var hbox = document.createElement("hbox");
			var tb = document.createElement("textbox");
			tb.setAttribute("style","width:12ex");
			tb.setAttribute("readonly","true");
			tb.setAttribute("value", cols[i][info[1]["name"][0]]);
			hbox.appendChild(tb);

			var tb = document.createElement("textbox");
			tb.setAttribute("style","width:10ex");
			tb.setAttribute("readonly","true");
			tb.setAttribute("value", cols[i][info[1]["type"][0]]);
			hbox.appendChild(tb);

			var tb = document.createElement("checkbox");
			tb.setAttribute("style","width:7ex");
			tb.setAttribute("disabled","true");
			if (cols[i][info[1]["pk"][0]] == 1)
				tb.setAttribute("checked", true);
			hbox.appendChild(tb);

			var tb = document.createElement("checkbox");
			tb.setAttribute("style","width:7ex");
			tb.setAttribute("disabled","true");
			if (cols[i][info[1]["notnull"][0]] != 0)
				tb.setAttribute("checked", true);
			hbox.appendChild(tb);

			var tb = document.createElement("textbox");
			tb.setAttribute("style","width:10ex");
			tb.setAttribute("readonly","true");
			var def_val = cols[i][info[1]["dflt_value"][0]];
			//if (info[2][i][info[1]["dflt_value"][0]] == 3)
			//	def_val = sm_quote(def_val);
			tb.setAttribute("value", def_val);
			hbox.appendChild(tb);

			var tb = document.createElement("button");
			tb.setAttribute("label", sm_getLStr("dropColumn"));
			tb.setAttribute("value", cols[i][info[1]["name"][0]]);
			tb.setAttribute("oncommand", "SQLiteManager.modifyTable('dropColumn','" + sTable + "','" + cols[i][info[1]["name"][0]] + "')");
      if (this.miDbObjects != 1) //no drop column option for master table
      	hbox.appendChild(tb);

			vbox.appendChild(hbox);
		}
		
		var aTableInfo = Database.getTableInfo(sTable, -1);

		document.getElementById("numFields").value = aTableInfo["numFields"];
		document.getElementById("numIndexes").value = aTableInfo["numIndexes"];
		document.getElementById("numRecords").value = aTableInfo["numRecords"];
  },

  printIndexInfo: function(sIndex) {
		var aIndexInfo = Database.getIndexInfo(sIndex, this.miDbObjects);
		
		document.getElementById("tabletoindex").value = aIndexInfo["table"];
		document.getElementById("fieldtoindex").value = aIndexInfo["cols"];
		document.getElementById("duplicatevalues").value = sm_getLStr("allowed");
		if(aIndexInfo["unique"] == 1)
			document.getElementById("duplicatevalues").value = sm_getLStr("notAllowed");
  },

	//loadTabBrowse: populates the table list and the tree view for current table
	//must be called whenever a database is opened/closed
	//and whenever the schema changes
	//depends entirely upon the values in "browse-type" and "browse-name" controls
  loadTabBrowse: function() {
    //no need to waste resources if this tab is not selected
    if(this.getSelectedTabId() != "tab-browse")
    	return false;
    	
    if (this.sCurrentDatabase == null)
    	return false;

		if (this.mostCurrObjType == null)
			return false;

		var sObjType = this.mostCurrObjType.toLowerCase();
		if (sObjType != "table" && sObjType != "view")
			return false;

		document.getElementById("browse-type").value = sObjType.toUpperCase();
		document.getElementById("browse-name").value = this.mostCurrObjName;

		//populate the treeview
    var sObjName = this.mostCurrObjName;
		if (sObjName != this.msBrowseObjName) {
			this.miOffset = 0;
			this.msBrowseObjName = sObjName;
			this.msBrowseCondition = "";
		}
			
		//some UI depends on whether table/master tables/view is shown
		var btnAdd =	document.getElementById("btnAddRecord");
		var btnEdit =	document.getElementById("btnEditRecord");
		var btnDelete =	document.getElementById("btnDeleteRecord");
		//two search buttons because set attribute cannot change oncommand attribute
		var btnSearchTable =	document.getElementById("btnSearchTable");
		var btnSearchView =	document.getElementById("btnSearchView");
		var treeChildren = document.getElementById("browse-treechildren");
		btnSearchTable.hidden = false;
		btnSearchView.hidden = true;

    var setting = [false, "menupopup-editTableRow", "SQLiteManager.operateOnTable('update')"];
		if (sObjType == "table" && (this.mostCurrObjName == "sqlite_master" || this.mostCurrObjName == "sqlite_temp_master")) {
      setting = [true, "menupopup-browse-copy", ""];
		}
		if (sObjType == "view") {
      setting = [true, "menupopup-browse-copy", ""];
			btnSearchView.hidden = false;
  		btnSearchTable.hidden = true;
		}

		btnAdd.disabled = setting[0];
		btnEdit.disabled = setting[0];
		btnDelete.disabled = setting[0];
		treeChildren.setAttribute("context", setting[1]);
		treeChildren.setAttribute("ondblclick", setting[2]);

    treeBrowse.ShowTable(false);

    try {
      var bRet = Database.loadTableData(sObjType, sObjName, this.msBrowseCondition,   
										this.miLimit, this.miOffset);
    } catch (e) { 
      sm_message(sm_getLStr("loadDataFailed"), 0x3);
			return false;
    }
    if (!bRet)
    	return bRet;

    var records = Database.getRecords();
    var columns = Database.getColumns();
  	this.miCount = Database.getTableCount(sObjName, this.msBrowseCondition);
    	
		this.manageNavigationControls();    	
    if (records && columns)
        treeBrowse.PopulateTableData(records, columns);
		return true;
  },

	onBrowseNavigate: function(sType) {
		switch(sType) {
			case "first":
				this.miOffset = 0;
				break;
			case "previous":
				this.miOffset = this.miOffset - this.miLimit;
				if (this.miOffset < 0)
					this.miOffset = 0;
				break;
			case "next":
				this.miOffset = this.miOffset + this.miLimit;
				break;
			case "last":
				this.miOffset = this.miCount - (this.miCount % this.miLimit);
				break;
		}
		this.loadTabBrowse();
	},
	
	manageNavigationControls: function() {
		//manage textboxes
  	document.getElementById("nav-total-val").value = this.miCount;
  	var iStart = (this.miCount == 0) ? 0 : (this.miOffset + 1);
  	document.getElementById("nav-start-val").value = iStart;
  	var iEnd = this.miOffset + this.miLimit;
  	iEnd = ((iEnd > this.miCount) || (this.miLimit == -1)) ? this.miCount : iEnd;
  	document.getElementById("nav-end-val").value = iEnd;

		//manage buttons
		var btnFirst = document.getElementById("btn-nav-first");
		var btnPrevious = document.getElementById("btn-nav-previous");
		var btnNext = document.getElementById("btn-nav-next");
		var btnLast = document.getElementById("btn-nav-last");

		btnFirst.disabled = false;
		btnPrevious.disabled = false;
		btnNext.disabled = false;
		btnLast.disabled = false;

    //manage the navigate buttons
    if (this.miLimit < 0 || this.miLimit >= this.miCount) {
			btnFirst.disabled = true;
			btnPrevious.disabled = true;
			btnNext.disabled = true;
			btnLast.disabled = true;
			return;
    }

		if (this.miOffset == 0) {
			btnFirst.disabled = true;
			btnPrevious.disabled = true;
		}
		else {
			btnFirst.disabled = false;
			btnPrevious.disabled = false;
		}
		if (this.miOffset + this.miLimit > this.miCount) {
			btnNext.disabled = true;
			btnLast.disabled = true;
		}
		else {
			btnNext.disabled = false;
			btnLast.disabled = false;
		}
	},
	
	//loadTabExecute: anything to be done when that tab is shown goes here
  loadTabExecute: function() {
    this.populateQueryListbox();
  },

	//loadTabDbInfo: anything to be done when that tab is shown goes here
  loadTabDbInfo: function() {
    //no need to waste resources if this tab is not selected
    if(this.getSelectedTabId() != "tab-dbinfo")
    	return false;
    	
    if (this.sCurrentDatabase == null)
    	return false;

    //auto_vacuum is a problem on Calendar 0.9 pre/Gecko 1.8.0.17pre
		var aSettings = ["auto_vacuum", "cache_size", "count_changes", "default_cache_size", "empty_result_callbacks", "encoding", "full_column_names", "fullfsync", "legacy_file_format", "locking_mode", "page_size", "max_page_count", "read_uncommitted", "short_column_names", "synchronous", "temp_store", "temp_store_directory"];
    if (gbGecko_1_9b3)
		  aSettings = ["schema_version", "user_version", "auto_vacuum", "cache_size", "count_changes", "default_cache_size", "empty_result_callbacks", "encoding", "full_column_names", "fullfsync", "legacy_file_format", "locking_mode", "page_size", "max_page_count", "read_uncommitted", "short_column_names", "synchronous", "temp_store", "temp_store_directory"];
		for(var i = 0; i < aSettings.length; i++)	{
		  var sSetting = aSettings[i];
			var node = document.getElementById("pr-" + sSetting);
			var newVal = Database.getSetting(sSetting);
			node.value = newVal;
		}
		return true;
	},

  searchView: function() {
  	var sViewName = this.aCurrObjNames["view"];
  	Database.loadTableData("view", sViewName, "", 1, 0);
    var records = Database.getRecords();
    if (records.length == 0) {
    	alert(sm_getLStr("noRecord"));
    	return false;
    }

		var columns = Database.getColumns();
  	var names = [], types = [];
    for (var col in columns) {
    	names[col] = columns[col][0];
    	types[col] = GetColumnTypeString(columns[col][1]);
    }
    var cols = [names, types];
    window.openDialog("chrome://sqlitemanager/content/RowOperations.xul",
					"RowOperations", "chrome, resizable, centerscreen, modal, dialog", 
					Database, this.aCurrObjNames["view"], "search-view", cols);
    return true;
  },

  //getSelectedTab: returns the selected tab
  getSelectedTab: function() {
		return document.getElementById("sm-tabs").selectedItem;
  },

  //getSelectedTabId: returns the id of the selected tab
  getSelectedTabId: function() {
		var oSelectedTab = document.getElementById("sm-tabs").selectedItem;
		var id = oSelectedTab.getAttribute("id");
		return id;
  },

  //doSelectTab: called when onselect event fires on tabs[id="sm-tabs"]
	doSelectTab: function(oSelectedTab) {
		var id = oSelectedTab.getAttribute("id");
		return this.loadTabWithId(id);
	},

  //selectStructTab: called when onselect event fires on tabs[id="sm-tabs-db"]
	selectStructTab: function(oSelectedTab) {
		var id = oSelectedTab.getAttribute("id");
		switch(id) {
			case "tab-db-norm":
				this.miDbObjects = 0;
				break;
			case "tab-db-mstr":
				this.miDbObjects = 1;
				break;
			case "tab-db-temp":
				this.miDbObjects = 2;
				break;
		}
		this.refreshDbStructure();
		return true;
	},

	loadTabWithId: function(sId) {
		switch(sId) {
			case "tab-structure":
				this.loadTabStructure();
				break;
			case "tab-browse":
				this.loadTabBrowse();
				break;
			case "tab-execute":
				this.loadTabExecute();
				break;
			case "tab-dbinfo":
				this.loadTabDbInfo();
				break;
		}
		return true;
	},

  //bImplicit: false = called from menuitem; true = function call
  useExtensionManagementTable: function(bUse, bImplicit) {
    var mi = document.getElementById("menu-general-extensionTable");

		if(this.sCurrentDatabase == null) {
		  //revert to the state before clicking
      mi.removeAttribute("checked");
			if (!bImplicit) alert(sm_getLStr("firstOpenADb"));
			return false;
		}

    smExtManager.setUsage(bUse, bImplicit);
    if (bUse) {
      mi.setAttribute("checked", "true");
      this.populateQueryListbox();
    }
    else
      mi.removeAttribute("checked");

		//refresh the structure tree here so that mgmt table is shown/removed
		this.refresh();

		//hide/show the images for query history in the execute sql tab
		document.getElementById("queryHistoryPrevImage").hidden = !bUse;
		document.getElementById("queryHistoryNextImage").hidden = !bUse;
		document.getElementById("querySaveByNameImage").hidden = !bUse;
		document.getElementById("queryHistoryClearImage").hidden = !bUse;

		document.getElementById("listbox-queries").hidden = !bUse;

    return true;
  },

  showPrevSql: function() {
    var sQuery = smExtManager.getPrevSql();
    if (!sQuery) return;
    document.getElementById("txtSqlStatement").value = sQuery;
  },

  showNextSql: function() {
    var sQuery = smExtManager.getNextSql();
    if (!sQuery) return;
    document.getElementById("txtSqlStatement").value = sQuery;
  },

	saveSqlByName: function()	{
		var sQuery = document.getElementById("txtSqlStatement").value;
		if (sQuery.length <= 0)
			alert("Nothing to save.");

    if (smExtManager.saveSqlByName(sQuery))
      this.populateQueryListbox();
	},

  clearSqlHistory: function() {
    smExtManager.clearSqlHistory();
  },

	onSelectQuery: function() {
    var sVal = document.getElementById("listbox-queries").value;
    if (sVal != sQuerySelectInstruction)
  		document.getElementById("txtSqlStatement").value = sVal;
	},

	populateQueryListbox: function() {
		var listbox = document.getElementById("listbox-queries");
    if (this.sCurrentDatabase == null) {
      listbox.hidden = true;
    	return false;
    }
		var aQueries = smExtManager.getQueryList();
		if (aQueries.length)
		  aQueries.unshift(sQuerySelectInstruction);
		else
		  aQueries = [sQuerySelectInstruction];
		var sDefault = listbox.selectedItem;
		if (sDefault != null)
      sDefault = sDefault.label;
		PopulateDropDownItems(aQueries, listbox, sDefault);
	},

	runSqlStatement: function(sType) {
		if(this.sCurrentDatabase == null)	{
			alert(sm_getLStr("firstOpenADb"));
			return;
		}

		//get the query string from an xul page
		var sQuery = document.getElementById("txtSqlStatement").value;
		
		var queries = sql_tokenizer(sQuery);
    if (queries.length == 0) {
			alert(sm_getLStr("writeSomeSql"));
			return;
		}
    var bRet = false;
//		if(sType == "select")
    if (queries.length == 1) {
      sQuery = queries[0];
			bRet = Database.selectQuery(sQuery);
			//Database.executeMultiple(sQuery);
      //store the query in config db
			if (bRet) smExtManager.addQuery(sQuery);
      //set this value so that query history is reset to latest query
      //that is previous will again begin from the latest query
  		smExtManager.goToLastQuery();
		}
		else {
			Database.executeTransaction(queries);
		}
		
		var aData = Database.getRecords();
		var aColumns = Database.getColumns();
		//display the last error in the textbox
		document.getElementById("sqlLastError").value = Database.getLastError();
		if (bRet)
		  sm_message(sm_getLStr("rowsReturned") + ": " + aData.length, 0x2);
		//document.getElementById("sqlRowsAffected").value = " ( " + aData.length + " rows returned )";

		//the following two lines must be before the code for tree
		//otherwise, it does not refresh the structure tree as expected
		this.refreshDbStructure();
		this.loadTabBrowse();
		
		treeExecute.ShowTable(false);
		if (bRet) {
		  treeExecute.PopulateTableData(aData, aColumns);
		}
	},

  newDatabase: function() {
  	var sExt = "." + this.maFileExt[0];
		//prompt for a file name
		var fname = prompt("Enter the database name (" + sExt + " will be automatically appended to the name)", "", "Enter the Database Name");

		//if cancelled, abort
		if (fname == "" || fname == null)
			return false;
		
		//append the extension to the chosen name
		fname += sExt;
		
		//let the user choose the folder for the new db file	
		var dir = sm_chooseDirectory(sm_getLStr("selectFolderForDb"));
		if (dir != null) {
			//access this new copied file
			var newfile = Components.classes["@mozilla.org/file/local;1"]
								.createInstance(Components.interfaces.nsILocalFile);
			newfile.initWithPath(dir.path);
			newfile.append(fname);
			
			//if the file already exists, alert user that existing file will be opened 
			if(newfile.exists()) {
				alert(sm_getLStr("dbFileExists"));
			}

    	//if another file is already open,
			//confirm from user that it should be closed
    	if(this.closeDatabase(false)) {
				//assign the new file (nsIFile) to the current database
				this.sCurrentDatabase = newfile;
				//if the file does not exist, openDatabase will create it 
				this.setDatabase(this.sCurrentDatabase);
				return true;
			}
		}
		return false;
  },

	//closeDatabase: 
  closeDatabase: function(bAlert) {
		//nothing to close if no database is already open    
  	if(this.sCurrentDatabase == null)	{
   		if(bAlert)
        alert(sm_getLStr("noOpenDb"));
			return true;
		}
			
   	//if another file is already open, confirm before closing
   	var answer = true;
 		if(bAlert)
  		answer = smPrompt.confirm(null, "SQLite Manager", sm_getLStr("confirmClose"));

  	if(!answer)
  		return false;

    //save StructureTreeState in ext mgmt table if mgmt is in use
    if (smExtManager.getUsage()) {
      smExtManager.setStructTreeState(smStructTrees[0].aExpandedNodes);
    }
		//make the current database as null and 
		//call setDatabase to do appropriate things
		this.sCurrentDatabase = null;
		this.setDatabase(this.sCurrentDatabase);
		return true;
	},
		
  copyDatabase: function() {
 		if(this.sCurrentDatabase == null) {
			alert(sm_getLStr("firstOpenADb"));
			return;
		}
    var sExt = "." + this.maFileExt[0];
		//prompt for a file name
		var fname = prompt("Enter the database name (" + sExt + " will be automatically appended to the name)", "", "Enter the Database Name");

		//if cancelled, abort
		if (fname == "" || fname == null)
			return;
		else
			fname += sExt;
			
		//let the user choose the folder for the new db file	
		//let the user choose the folder for the new db file	
		var dir = sm_chooseDirectory(sm_getLStr("selectFolderForDb"));
		if (dir != null) {
			//copy the opened file to chosen location
			this.sCurrentDatabase.copyTo(dir, fname);

			//access this new copied file
			var newfile = Components.classes["@mozilla.org/file/local;1"]
								.createInstance(Components.interfaces.nsILocalFile);
			newfile.initWithPath(dir.path);
			newfile.append(fname);
			
			//if the file does not exist, openDatabase will create it 
			if(!newfile.exists()) {
				var ans = smPrompt.confirm(null, "SQLite Manager", sm_getLStr("copyFailed"));
				if(!ans)
					return;
			}

			//assign the new file (nsIFile) to the current database
    	if(this.closeDatabase(false)) {
				this.sCurrentDatabase = newfile;
        this.setDatabase(this.sCurrentDatabase);
        return;
			}
		}
		return;
  },
    
  compactDatabase: function() {
    if (this.sCurrentDatabase == null) {
			alert(sm_getLStr("firstOpenADb"));
			return false;
		}
  	var sQuery = "VACUUM";
  	//cannot vacuum from within a transaction
  	Database.selectQuery(sQuery);
  	return true;
  },

  analyzeDatabase: function() {
    if (this.sCurrentDatabase == null) {
			alert(sm_getLStr("firstOpenADb"));
			return false;
    }
  	var sQuery = "ANALYZE";
  	//cannot vacuum from within a transaction
  	Database.selectQuery(sQuery);
  	return true;
  },

  checkIntegrity: function() {
    if (this.sCurrentDatabase == null) {
			alert(sm_getLStr("firstOpenADb"));
			return false;
    }
    Database.selectQuery("PRAGMA integrity_check");
    var records = Database.getRecords();
    var columns = Database.getColumns();

    var txt = sm_getLStr("integrityResultPrefix") + ": ";
    //report OK if i row returned containing the value "ok"
    if (records.length == 1 && records[0][0] == "ok")
      alert(txt + sm_getLStr("ok"));
    else
      alert(txt + sm_getLStr("notOk"));
    return true;
  },

  openDatabase: function() {        

		const nsIFilePicker = Components.interfaces.nsIFilePicker;
		var fp = Components.classes["@mozilla.org/filepicker;1"]
			           .createInstance(nsIFilePicker);
		fp.init(window, sm_getLStr("selectDb"), nsIFilePicker.modeOpen);
    var sExt = "";
		for (var iCnt = 0; iCnt < this.maFileExt.length; iCnt++) {
      sExt += "*." + this.maFileExt[iCnt] + ";";
		}
	  fp.appendFilter(sm_getLStr("sqliteDbFiles") + " (" + sExt + ")", sExt);
		fp.appendFilters(nsIFilePicker.filterAll);
		
		var rv = fp.show();
		if (rv == nsIFilePicker.returnOK || rv == nsIFilePicker.returnReplace) {
		  // work with returned nsILocalFile...
    	if(this.closeDatabase(false)) {
				this.sCurrentDatabase = fp.file;
        this.setDatabase(this.sCurrentDatabase);
        return true;
			}
		}
		return false;
  },

  saveDatabase: function() {        
  },

  createTable: function() {        
    if (this.sCurrentDatabase != null) {
      window.openDialog("chrome://sqlitemanager/content/createTable.xul",
							"createTable", "chrome, resizable, centerscreen, modal, dialog", 
							Database, null);
			this.refreshDbStructure();
			this.loadTabBrowse();
		}
		else {
			alert(sm_getLStr("firstOpenADb"));
		}
  },

  modifyTable: function(sOperation, sTable, sColumn) {        
  	if(sOperation == "dropColumn") {
			var bReturn = CreateManager.modifyTable("dropColumn", sTable, sColumn);
			if(bReturn) {
				this.refreshDbStructure();
				this.loadTabBrowse();
			}
			return bReturn;
		}
		return false;
  },

  reindexIndex: function() {        
  	var sCurrIndex = this.aCurrObjNames["index"];
  	if(sCurrIndex != null && sCurrIndex != undefined && sCurrIndex.length > 0) {
			var bReturn = Database.reindexObject("INDEX", sCurrIndex);
			return bReturn;
		}
		return false;
  },

	dropObject: function(sObjectType) {
    if (this.sCurrentDatabase == null) {
			alert(sm_getLStr("firstOpenADb"));
			return false;
		}

		var sObjectName = "";
		sObjectName = this.aCurrObjNames[sObjectType];
		
		var aNames = this.aObjNames[sObjectType];

		if(aNames.length == 0) {
			alert(sm_getLStr("noObjectToDelete") + ": " + sObjectType);
			return false;
		}
		var bReturn = Database.dropObject(sObjectType, sObjectName);
		if(bReturn) {
			sm_message(sm_getLStr("dropDone"), 0x2);
			this.refreshDbStructure();
			this.loadTabBrowse();
		}
		return bReturn;
  },

	importFromFile: function() {
    if (this.sCurrentDatabase == null) {
			alert(sm_getLStr("firstOpenADb"));
			return false;
		}
    window.openDialog("chrome://sqlitemanager/content/export.xul",
						"import",	"chrome, resizable, centerscreen, modal, dialog", 
						Database, "import");
		this.refreshDbStructure();
		this.loadTabBrowse();
		return true;
	},
	
	exportObject: function(sObjectType) {
    if (this.sCurrentDatabase == null) {
			alert(sm_getLStr("firstOpenADb"));
			return false;
		}

		var sObjectName = "";
		sObjectName = this.aCurrObjNames[sObjectType];
		
		var aNames = this.aObjNames[sObjectType];

		if(aNames.length == 0) {
			alert("No " + sObjectType + " to export");
			return false;
		}
    window.openDialog("chrome://sqlitemanager/content/export.xul",
						"export",	"chrome, resizable, centerscreen, modal, dialog", 
						Database, sObjectType, sObjectName);
		return true;
  },

  createObject: function(sObjectType) {
    if (this.sCurrentDatabase == null) {
			alert(sm_getLStr("firstOpenADb"));
			return false;
		}

		var xul = "chrome://sqlitemanager/content/create" + sObjectType + ".xul";
    var ret = window.openDialog(xul, "create" + sObjectType, 
						"chrome, resizable, centerscreen, modal, dialog", 
						Database, this.aCurrObjNames["table"], sObjectType);
		this.refreshDbStructure();
		this.loadTabBrowse();
		return true;
  },

	copyTable: function(sTableName) {
		var check = {value: false};   // default the checkbox to false
		var input = {value: ""};   // default the edit field to Bob
		var result = smPrompt.prompt(null, "Copy Table " + sTableName,
		                  "Enter the new name of the table", input, 
											"Only Structure (data will not be copied)", 
											check);
		var sNewTable = input.value;
		sNewTable = sm_backquote(sNewTable);
		var bOnlyStructure = check.value;
		//returns true on OK, false on cancel
		if (!result || sNewTable.length == 0)
		  return false;
		  
		var info = Database.getMasterInfo(sTableName, ["sql"], this.miDbObjects);
		var r_sql = info[0];

		var sTab = sm_backquote(sTableName);	
		var sTempTable = g_tempNamePrefix + sTableName;
		sTempTable = sm_backquote(sTempTable);
		var aQueries = [];
		aQueries.push("ALTER TABLE " + sTab + " RENAME TO "	+ sTempTable);
		aQueries.push(r_sql);
		aQueries.push("ALTER TABLE " + sTab + " RENAME TO "	+ sNewTable);
		aQueries.push("ALTER TABLE " + sTempTable + " RENAME TO "	+ sTab);
		if(!bOnlyStructure) {
			aQueries.push("INSERT INTO " + sNewTable + " SELECT * FROM " + sTab);
		}
	
		return Database.confirmAndExecute(aQueries, "Copy the Table: " + sTableName);
	},

	renameTable: function(sTableName)	{
		var check = {value: false};   // default the checkbox to false
		var input = {value: ""};   // default the edit field to Bob
		var result = smPrompt.prompt(null, "Rename Table " + sTableName,
		                  "Enter the new name of the table", 
		                  input, null, check);
		var sNewTable = input.value;
		sNewTable = sm_backquote(sNewTable);
		//returns true on OK, false on cancel
		if (!result || sNewTable.length == 0)
		  return false;
		  
		var sTab = sm_backquote(sTableName);	
		var sQuery = "ALTER TABLE " + sTab + " RENAME TO " + sNewTable;
		
		return Database.confirmAndExecute([sQuery], "Rename table " + sTableName);
	},

// operateOnTable: various operations on a given table
// sOperation = rename | copy | reindex | delete	| 
//              insert | search | update
  operateOnTable: function(sOperation) {
		//these operations make sense in the context of some table
		//so, take action only if there is a valid selcted db and table
	  if (this.sCurrentDatabase == null || this.aCurrObjNames["table"] == null) {
			alert(sm_getLStr("noDbOrTable"));
			return false;
		}
  	var sCurrTable = this.aCurrObjNames["table"];
  	var bReturn = false;
  	var bRefresh = false; //to reload tabs
		switch(sOperation) {
			case "reindex":
				return Database.reindexObject("TABLE", sCurrTable);
				break;
			case "analyze":
			  return Database.analyzeTable(sCurrTable);
				break;
		}
  	if(sOperation == "copy") {
			var bReturn = this.copyTable(sCurrTable);
			if(bReturn)	this.refresh();
			return bReturn;
		}
  	if(sOperation == "rename") {
			var bReturn = this.renameTable(sCurrTable);
			if(bReturn)	this.refresh();
			return bReturn;
		}
  	if(sOperation == "drop") {
			var bReturn = Database.dropObject("TABLE", sCurrTable);
			if(bReturn)	this.refresh();
			return bReturn;
		}
  	if(sOperation == "empty") {
			var bReturn = Database.emptyTable(sCurrTable);
			if(bReturn)	this.refresh();
			return bReturn;
		}
  	if(sOperation == "addColumn") {
  		var newCol = [];
			newCol["name"] = document.getElementById("tb-addcol-name").value;
			newCol["type"] = document.getElementById("tb-addcol-type").value;
			newCol["notnull"] = document.getElementById("tb-addcol-notnull").checked;
			newCol["dflt_value"] = document.getElementById("tb-addcol-default").value;
			newCol["dflt_value"] = sm_makeSqlValue(newCol["dflt_value"]);
			var bReturn = CreateManager.modifyTable("addColumn", sCurrTable, newCol);
			if(bReturn) {
				document.getElementById("tb-addcol-name").value = "";
				document.getElementById("tb-addcol-type").value = "";
				document.getElementById("tb-addcol-notnull").checked = false;
				document.getElementById("tb-addcol-default").value = "";
				this.refresh();
			}
			return bReturn;
		}
			
		//update the first selected row in the tree, else alert to select
  	//if selection exists, pass the rowid as the last arg of openDialog
  	var aRowIds = [];
  	var rowCriteria = "";
  	if(sOperation == "update" || sOperation == "delete") {
  		var colMain = Database.getTableRowidCol(this.aCurrObjNames["table"]);
			colMain["name"] = "`" + colMain["name"] + "`";

  		//allowing for multiple selection in the tree
			var tree = document.getElementById("browse-tree");
			var start = new Object();
			var end = new Object();
			var numRanges = tree.view.selection.getRangeCount();

			for (var t = 0; t < numRanges; t++) {
				tree.view.selection.getRangeAt(t,start,end);
			  for (var v = start.value; v <= end.value; v++) {
			  	var rowid = tree.view.getCellText(v,
							tree.columns.getColumnAt(colMain["cid"]));
					aRowIds.push(rowid);
			  }
			}
			//do nothing, if nothing is selected
			if(aRowIds.length == 0)	{
				alert(sm_getLStr("noRecord"));
				return false;
			}
			//if editing, should select only one record
			if (sOperation == "update")	{
				if (aRowIds.length != 1) {
					alert(sm_getLStr("onlyOneRecord"));
					return false;
				}
				rowCriteria = " " + colMain["name"] + " = " + aRowIds[0];
			}
			//if deleting, pass as argument rowid of all selected records to delete
			if (sOperation == "delete") {
    		var criteria = colMain["name"] + " IN (" + aRowIds.toString() + ")";
    		var sQuery = "DELETE FROM " + sm_backquote(sCurrTable) + " WHERE " + criteria;
    		//IMPORTANT: the last parameter is totally undocumented.
				var bReturn = Database.confirmAndExecute([sQuery], ["Delete " + aRowIds.length + " records from " + sCurrTable, false]);
				if(bReturn)
					this.loadTabBrowse();
				return bReturn;
			}
		}
    window.openDialog("chrome://sqlitemanager/content/RowOperations.xul",
					"RowOperations", "chrome, resizable, centerscreen, modal, dialog", 
					Database, this.aCurrObjNames["table"], sOperation, rowCriteria);
		if(sOperation != "search") {
			if(sOperation != "update") {
				this.refreshDbStructure();
			}
			this.loadTabBrowse();
		}
		return true;
  },

  // populateProfileDBList: Load list of files with default file extensions
  populateProfileDBList: function() {
    // Get the nsIFile object pointing to the profile directory
    var fileList = Components.classes["@mozilla.org/file/directory_service;1"]
            .getService(Components.interfaces.nsIProperties)
            .get("ProfD", Components.interfaces.nsIFile)
            .directoryEntries; 
    var file;
    var iFileCount = 0;
    
    var aItems = new Array();
    aItems.push(sm_getLStr("selectProfileDb"));

    var aSplit, sExt;
		while (fileList.hasMoreElements()) {
      file = fileList.getNext().QueryInterface(Components.interfaces.nsIFile);
      aSplit = file.leafName.split(".");
      sExt = aSplit[aSplit.length - 1]; 

      if (this.maFileExt.indexOf(sExt) != -1) {
        iFileCount = iFileCount + 1;
        aItems.push(file.leafName);
      } 
    }
    //get the node for the popup menus to show profile db list
    var listboxProfileDB = document.getElementById("listbox-profileDB");

    PopulateDropDownItems(aItems, listboxProfileDB, null);
    sm_message(sm_getLStr("filesInProfileDbList") + ": " + iFileCount, 0x2);
  },

  // openProfileDatabase: open a file from the database dropdown list
  openProfileDatabase: function() { 
		//get the node for dropdown menu in which profile db list is shown       
  	var listboxProfileDB = document.getElementById("listbox-profileDB");
    var sLabel = listboxProfileDB.value;

    var file = Components.classes["@mozilla.org/file/directory_service;1"]
            .getService(Components.interfaces.nsIProperties)
            .get("ProfD", Components.interfaces.nsIFile);
    file.append(sLabel);
		
		//proceed only if the file exists
		//we are in the profile folder via the listbox, so open if the file exists
		//do not attempt to create new file
		if(!file.exists()) {
			alert(sm_getLStr("invalidProfileDb"));
			return false;
		}
  	if(this.closeDatabase(false))	{
			this.sCurrentDatabase = file;
      this.setDatabase(this.sCurrentDatabase);
      return true;
		}
		return false;
	},

  // setDatabase: set the current database to nsiFileObj
  setDatabase: function(nsiFileObj) {
  //when passed as arg, works but fails to show .path and .leafName properties
//      this.sCurrentDatabase = nsiFileObj; 

    this.mbDbJustOpened = true;

		treeBrowse.ShowTable(false);
		treeExecute.ShowTable(false);

		//try connecting to database
		var bConnected = false;
		try	{
			if(this.sCurrentDatabase != null) {
			 if (this.mbSharedPagerCache)
  		    bConnected = Database.openDatabase(this.sCurrentDatabase);
			 else
	   	    bConnected = Database.openUnsharedDatabase(this.sCurrentDatabase);

    		document.getElementById("vb-structureTab").hidden = false;
    		document.getElementById("vb-browseTab").hidden = false;
    		document.getElementById("vb-executeTab").hidden = false;
    		document.getElementById("vb-dbInfoTab").hidden = false;
		  }
			if(this.sCurrentDatabase == null) {
		    Database.closeConnection();
    		//call it to hide all things there - Issue #90, etc.
        this.emptyTabStructure();
     		document.getElementById("vb-structureTab").hidden = true;
    		document.getElementById("vb-browseTab").hidden = true;
    		document.getElementById("vb-executeTab").hidden = true;
    		document.getElementById("vb-dbInfoTab").hidden = true;
        this.useExtensionManagementTable(false, true);
		  }
		} 
		catch (e)	{
	    var sTemp = this.sCurrentDatabase.path;
	    this.sCurrentDatabase = null;
	    sm_message("Connect to '" + sTemp + "' failed: " + e, 0x3);
	    return;
		}

    var path = "", leafName = "";
    if (nsiFileObj != null && bConnected) {
      path = nsiFileObj.path;
      leafName = nsiFileObj.leafName;
      //change the mruPath.1 in preferences
      sm_setPreferenceValue("mruPath.1", path);

      //extension related mgmt info
      if (this.m_bMgmtDataIsInDb) {
        smExtManager = new SMExtensionManager();
  //  else
  //    smExtManager = new SMExtensionManagerExternal();
  
        this.useExtensionManagementTable(smExtManager.getUsage(), true);
      }

      //display the sqlite version in the status bar
      var sV = sm_getLStr("sqliteVersion") + ": " + Database.sqliteVersion;
      document.getElementById("sbSqliteVersion").setAttribute("label",sV);
    }
    if (!bConnected) {
    	this.sCurrentDatabase = null;
    }
    //change window title to show db file path
		document.title = sm_getLStr("extName") + " - " + path;
		document.getElementById("tb-dbName").value = leafName;

		//reload the two tabs
    this.refreshDbStructure();

    this.mbDbJustOpened = false;
  },
    
  setSqlText: function(val) {
    document.getElementById("txtSqlStatement").value = val;
  },

  setSharedPagerCache: function() {
    var mi = document.getElementById("menu-general-sharedPagerCache");
    this.mbSharedPagerCache = mi.hasAttribute("checked");
    if (!gbGecko_1_9) {//sqlite 3.5.4.1 onwards supports openUnsharedDatabase
      mi.hidden = true;
    }
  }
};

//observer for something
function smExampleObserver() {
  this.register();
}

smExampleObserver.prototype = 
{
  observe: function(subject, topic, data) {
     // Do your stuff here.
  },
  
	register: function() {
    var observerService = Components.classes["@mozilla.org/observer-service;1"]
                          .getService(Components.interfaces.nsIObserverService);
    observerService.addObserver(this, "myTopicID", false);
  },
  
	unregister: function() {
    var observerService = Components.classes["@mozilla.org/observer-service;1"]
                            .getService(Components.interfaces.nsIObserverService);
    observerService.removeObserver(this, "myTopicID");
  }
}